#!/usr/bin/perl -w

# mailgraph -- a postfix statistics rrdtool frontend
# copyright (c) 2000, 2001 David Schweikert <dws@ee.ethz.ch>
# released under the GNU General Public License

use RRDs;

my $VERSION = 0.14;

my $host = `hostname`; chomp $host;
my $xpoints = 540;
my $ypoints = 150;
my $ypoints_err = 80;
my $imgs_dir = 'imgs'; # where to store the images
my $imgs_url = 'imgs'; # where to reference the images in HTML (<IMG> tag)
my $rrd = 'mailgraph.rrd';

BEGIN {
    print "Content-Type: text/html\n\n";
    $SIG{__WARN__} = sub { print "<pre>Warning: ".(shift)."</pre>"; };
    $SIG{__DIE__} = sub { print "<pre>Error: ".(shift)."</pre>"; exit 1 }
}

sub graph($$$)
{
	my $range = shift;
	my $file = shift;
	my $title = shift;
	my $img_file = "$imgs_dir/${file}.png";
	my $img_url = "$imgs_url/${file}.png";

	my ($graphret,$xs,$ys) = RRDs::graph($img_file,
		'--imgformat', 'PNG',
		'--width', $xpoints,
		'--height', $ypoints,
		'--start', "-$range",
		'--end', "-".int($range*0.01),
		'--vertical-label', 'msgs/hour',
		'--title', $title,
		'--lazy',
		"DEF:sent=$rrd:sent:AVERAGE",
		"DEF:recv=$rrd:recv:AVERAGE",
		"DEF:msent=$rrd:sent:MAX",
		"DEF:mrecv=$rrd:recv:MAX",
		"CDEF:rsent=sent,60,*",
		"CDEF:rrecv=recv,60,*",
		"CDEF:rmsent=msent,60,*",
		"CDEF:rmrecv=mrecv,60,*",
		"CDEF:vsent=sent,UN,0,sent,IF,$range,*",
		"CDEF:vrecv=recv,UN,0,recv,IF,$range,*",
		'AREA:rsent#000099:Sent',
		'GPRINT:vsent:AVERAGE:total\: %.0lf msgs',
		'GPRINT:rmsent:MAX:max\: %.0lf msgs/min',
		'LINE2:rrecv#009900:Received',
		'GPRINT:vrecv:AVERAGE:total\: %.0lf msgs',
		'GPRINT:rmrecv:MAX:max\: %.0lf msgs/min\l',
		'HRULE:0#000000',
		'COMMENT:\s',
		'COMMENT:Graph Created on '.localtime(time).'\r',
	);
	return RRDs::error || "<IMG BORDER=\"0\" WIDTH=\"$xs\" HEIGHT=\"$ys\" SRC=\"$img_url\" ALT=\"mailgraph\">";

}


sub graph_err($$$)
{
	my $range = shift;
	my $file = shift;
	my $title = shift;
	my $img_file = "$imgs_dir/${file}_err.png";
	my $img_url = "$imgs_url/${file}_err.png";

	my ($graphret,$xs,$ys) = RRDs::graph($img_file,
		'--imgformat', 'PNG',
		'--width', $xpoints,
		'--height', $ypoints_err,
		'--start', "-$range",
		'--end', "-".int($range*0.01),
		'--vertical-label', 'msgs/hour',
		'--title', "$title of Errors",
		'--lazy',
		"DEF:bounced=$rrd:bounced:AVERAGE",
		"DEF:rejected=$rrd:rejected:AVERAGE",
		"DEF:mbounced=$rrd:bounced:MAX",
		"DEF:mrejected=$rrd:rejected:MAX",
		"CDEF:rbounced=bounced,3600,*",
		"CDEF:rrejected=rejected,3600,*",
		"CDEF:vbounced=bounced,UN,0,bounced,IF,$range,*",
		"CDEF:vrejected=rejected,UN,0,rejected,IF,$range,*",
		"CDEF:rmbounced=mbounced,60,*",
		"CDEF:rmrejected=mrejected,60,*",
		'AREA:rbounced#000000:Bounced',
		'GPRINT:vbounced:AVERAGE:total\: %.0lf msgs',
		'GPRINT:rmbounced:MAX:max\: %.0lf msgs/min',
		'LINE2:rrejected#BB0000:Rejected',
		'GPRINT:vrejected:AVERAGE:total\: %.0lf msgs',
		'GPRINT:rmrejected:MAX:max\: %.0lf msgs/min\l',
		'HRULE:0#000000',
		'COMMENT:\s',
		'COMMENT:Graph Created on '.localtime(time).'\r',
	);
	return RRDs::error || "<IMG BORDER=\"0\" WIDTH=\"$xs\" HEIGHT=\"$ys\" SRC=\"$img_url\" ALT=\"mailgraph\">";
}

print <<HEADER;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML>
<HEAD>
<TITLE>Mail Statistics for $host</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">
HEADER

my $day = 3600*24;
my $week = $day*7;
my $month = $day*31;
my $year = $day*365;

print "<H1>Mail Statistics for $host</H1>\n";
print "<H2>Day Stats</H2>\n";
print '<P>', graph($day, "mailgraph_day", "Day Graph"), "\n";
print '<P>', graph_err($day, "mailgraph_day", "Day Graph"), "\n";
print "<H2>Week Stats</H2>\n";
print '<P>', graph($week, "mailgraph_week", "Week Graph"), "\n";
print '<P>', graph_err($week, "mailgraph_week", "Week Graph"), "\n";
print "<H2>Month Stats</H2>\n";
print '<P>', graph($month, "mailgraph_month", "Month Graph"), "\n";
print '<P>', graph_err($month, "mailgraph_month", "Month Graph"), "\n";
print "<H2>Year Stats</H2>\n";
print '<P>', graph($year, "mailgraph_year", "Year Graph"), "\n";
print '<P>', graph_err($year, "mailgraph_year", "Year Graph"), "\n";

print <<FOOTER;
<table border="0" width="400"><tr><td align="left">
<A href="http://people.ee.ethz.ch/~dws/software/mailgraph">Mailgraph</A> $VERSION
by <A href="http://people.ee.ethz.ch/~dws/">David Schweikert</A></td>
<td ALIGN="right">
<a HREF="http://people.ee.ethz.ch/~oetiker/webtools/rrdtool/"><img border="0" src="http://people.ethz.ch/~oetiker/webtools/rrdtool/.pics/rrdtool.gif" alt="" width="120" height="34"></a>
</td></tr></table>
</BODY>\n
FOOTER
